@@ -1,106 +1,165 @@
-#' SHAP contribution dependency plots
+#' SHAP dependence plots
 #'
-#' Visualizing the SHAP feature contribution to prediction dependencies on feature value.
+#' Visualizes SHAP values against feature values to gain an impression of feature effects.
 #'
-#' @param data data as a \code{matrix} or \code{dgCMatrix}.
-#' @param shap_contrib a matrix of SHAP contributions that was computed earlier for the above
-#'          \code{data}. When it is NULL, it is computed internally using \code{model} and \code{data}.
-#' @param features a vector of either column indices or of feature names to plot. When it is NULL,
-#'          feature importance is calculated, and \code{top_n} high ranked features are taken.
-#' @param top_n when \code{features} is NULL, top_n [1, 100] most important features in a model are taken.
-#' @param model an \code{xgb.Booster} model. It has to be provided when either \code{shap_contrib}
-#'          or \code{features} is missing.
-#' @param trees passed to \code{\link{xgb.importance}} when \code{features = NULL}.
-#' @param target_class is only relevant for multiclass models. When it is set to a 0-based class index,
-#'          only SHAP contributions for that specific class are used.
-#'          If it is not set, SHAP importances are averaged over all classes.
-#' @param approxcontrib passed to \code{\link{predict.xgb.Booster}} when \code{shap_contrib = NULL}.
-#' @param subsample a random fraction of data points to use for plotting. When it is NULL,
-#'          it is set so that up to 100K data points are used.
-#' @param n_col a number of columns in a grid of plots.
-#' @param col color of the scatterplot markers.
-#' @param pch scatterplot marker.
-#' @param discrete_n_uniq a maximal number of unique values in a feature to consider it as discrete.
-#' @param discrete_jitter an \code{amount} parameter of jitter added to discrete features' positions.
-#' @param ylab a y-axis label in 1D plots.
-#' @param plot_NA whether the contributions of cases with missing values should also be plotted.
-#' @param col_NA a color of marker for missing value contributions.
-#' @param pch_NA a marker type for NA values.
-#' @param pos_NA a relative position of the x-location where NA values are shown:
-#'          \code{min(x) + (max(x) - min(x)) * pos_NA}.
-#' @param plot_loess whether to plot loess-smoothed curves. The smoothing is only done for features with
-#'          more than 5 distinct values.
-#' @param col_loess a color to use for the loess curves.
-#' @param span_loess the \code{span} parameter in \code{\link[stats]{loess}}'s call.
-#' @param which whether to do univariate or bivariate plotting. NOTE: only 1D is implemented so far.
-#' @param plot whether a plot should be drawn. If FALSE, only a list of matrices is returned.
-#' @param ... other parameters passed to \code{plot}.
+#' @param data The data to explain as a `matrix` or `dgCMatrix`.
+#' @param shap_contrib Matrix of SHAP contributions of `data`.
+#'        The default (`NULL`) computes it from `model` and `data`.
+#' @param features Vector of column indices or feature names to plot.
+#'        When `NULL` (default), the `top_n` most important features are selected
+#'        by [xgb.importance()].
+#' @param top_n How many of the most important features (<= 100) should be selected?
+#'        By default 1 for SHAP dependence and 10 for SHAP summary).
+#'        Only used when `features = NULL`.
+#' @param model An `xgb.Booster` model. Only required when `shap_contrib = NULL` or
+#'        `features = NULL`.
+#' @param trees Passed to [xgb.importance()] when `features = NULL`.
+#' @param target_class Only relevant for multiclass models. The default (`NULL`)
+#'        averages the SHAP values over all classes. Pass a (0-based) class index
+#'        to show only SHAP values of that class.
+#' @param approxcontrib Passed to `predict()` when `shap_contrib = NULL`.
+#' @param subsample Fraction of data points randomly picked for plotting.
+#'        The default (`NULL`) will use up to 100k data points.
+#' @param n_col Number of columns in a grid of plots.
+#' @param col Color of the scatterplot markers.
+#' @param pch Scatterplot marker.
+#' @param discrete_n_uniq Maximal number of unique feature values to consider the
+#'        feature as discrete.
+#' @param discrete_jitter Jitter amount added to the values of discrete features.
+#' @param ylab The y-axis label in 1D plots.
+#' @param plot_NA Should contributions of cases with missing values be plotted?
+#'        Default is `TRUE`.
+#' @param col_NA Color of marker for missing value contributions.
+#' @param pch_NA Marker type for `NA` values.
+#' @param pos_NA Relative position of the x-location where `NA` values are shown:
+#'        `min(x) + (max(x) - min(x)) * pos_NA`.
+#' @param plot_loess Should loess-smoothed curves be plotted? (Default is `TRUE`).
+#'        The smoothing is only done for features with more than 5 distinct values.
+#' @param col_loess Color of loess curves.
+#' @param span_loess The `span` parameter of [stats::loess()].
+#' @param which Whether to do univariate or bivariate plotting. Currently, only "1d" is implemented.
+#' @param plot Should the plot be drawn? (Default is `TRUE`).
+#'        If `FALSE`, only a list of matrices is returned.
+#' @param ... Other parameters passed to [graphics::plot()].
 #'
 #' @details
 #'
 #' These scatterplots represent how SHAP feature contributions depend of feature values.
-#' The similarity to partial dependency plots is that they also give an idea for how feature values
-#' affect predictions. However, in partial dependency plots, we usually see marginal dependencies
-#' of model prediction on feature value, while SHAP contribution dependency plots display the estimated
-#' contributions of a feature to model prediction for each individual case.
+#' The similarity to partial dependence plots is that they also give an idea for how feature values
+#' affect predictions. However, in partial dependence plots, we see marginal dependencies
+#' of model prediction on feature value, while SHAP dependence plots display the estimated
+#' contributions of a feature to the prediction for each individual case.
 #'
-#' When \code{plot_loess = TRUE} is set, feature values are rounded to 3 significant digits and
-#' weighted LOESS is computed and plotted, where weights are the numbers of data points
+#' When `plot_loess = TRUE`, feature values are rounded to three significant digits and
+#' weighted LOESS is computed and plotted, where the weights are the numbers of data points
 #' at each rounded value.
 #'
-#' Note: SHAP contributions are shown on the scale of model margin. E.g., for a logistic binomial objective,
-#' the margin is prediction before a sigmoidal transform into probability-like values.
+#' Note: SHAP contributions are on the scale of the model margin.
+#' E.g., for a logistic binomial objective, the margin is on log-odds scale.
 #' Also, since SHAP stands for "SHapley Additive exPlanation" (model prediction = sum of SHAP
 #' contributions for all features + bias), depending on the objective used, transforming SHAP
 #' contributions for a feature from the marginal to the prediction space is not necessarily
 #' a meaningful thing to do.
 #'
 #' @return
-#'
-#' In addition to producing plots (when \code{plot=TRUE}), it silently returns a list of two matrices:
-#' \itemize{
-#'  \item \code{data} the values of selected features;
-#'  \item \code{shap_contrib} the contributions of selected features.
-#' }
+#' In addition to producing plots (when `plot = TRUE`), it silently returns a list of two matrices:
+#' - `data`: Feature value matrix.
+#' - `shap_contrib`: Corresponding SHAP value matrix.
 #'
 #' @references
-#'
-#' Scott M. Lundberg, Su-In Lee, "A Unified Approach to Interpreting Model Predictions", NIPS Proceedings 2017, \url{https://arxiv.org/abs/1705.07874}
-#'
-#' Scott M. Lundberg, Su-In Lee, "Consistent feature attribution for tree ensembles", \url{https://arxiv.org/abs/1706.06060}
+#' 1. Scott M. Lundberg, Su-In Lee, "A Unified Approach to Interpreting Model Predictions",
+#'    NIPS Proceedings 2017, <https://arxiv.org/abs/1705.07874>
+#' 2. Scott M. Lundberg, Su-In Lee, "Consistent feature attribution for tree ensembles",
+#'    <https://arxiv.org/abs/1706.06060>
 #'
 #' @examples
 #'
-#' data(agaricus.train, package='xgboost')
-#' data(agaricus.test, package='xgboost')
+#' data(agaricus.train, package = "xgboost")
+#' data(agaricus.test, package = "xgboost")
 #'
-#' bst <- xgboost(agaricus.train$data, agaricus.train$label, nrounds = 50,
-#'                eta = 0.1, max_depth = 3, subsample = .5,
-#'                method = "hist", objective = "binary:logistic", nthread = 2, verbose = 0)
+#' ## Keep the number of threads to 1 for examples
+#' nthread <- 1
+#' data.table::setDTthreads(nthread)
+#' nrounds <- 20
+#'
+#' bst <- xgboost(
+#'   agaricus.train$data,
+#'   agaricus.train$label,
+#'   nrounds = nrounds,
+#'   eta = 0.1,
+#'   max_depth = 3,
+#'   subsample = 0.5,
+#'   objective = "binary:logistic",
+#'   nthread = nthread,
+#'   verbose = 0
+#' )
 #'
 #' xgb.plot.shap(agaricus.test$data, model = bst, features = "odor=none")
+#'
 #' contr <- predict(bst, agaricus.test$data, predcontrib = TRUE)
 #' xgb.plot.shap(agaricus.test$data, contr, model = bst, top_n = 12, n_col = 3)
-#' xgb.ggplot.shap.summary(agaricus.test$data, contr, model = bst, top_n = 12)  # Summary plot
 #'
-#' # multiclass example - plots for each class separately:
+#' # Summary plot
+#' xgb.ggplot.shap.summary(agaricus.test$data, contr, model = bst, top_n = 12)
+#'
+#' # Multiclass example - plots for each class separately:
 #' nclass <- 3
-#' nrounds <- 20
 #' x <- as.matrix(iris[, -5])
 #' set.seed(123)
 #' is.na(x[sample(nrow(x) * 4, 30)]) <- TRUE # introduce some missing values
-#' mbst <- xgboost(data = x, label = as.numeric(iris$Species) - 1, nrounds = nrounds,
-#'                 max_depth = 2, eta = 0.3, subsample = .5, nthread = 2,
-#'                 objective = "multi:softprob", num_class = nclass, verbose = 0)
-#' trees0 <- seq(from=0, by=nclass, length.out=nrounds)
+#'
+#' mbst <- xgboost(
+#'   data = x,
+#'   label = as.numeric(iris$Species) - 1,
+#'   nrounds = nrounds,
+#'   max_depth = 2,
+#'   eta = 0.3,
+#'   subsample = 0.5,
+#'   nthread = nthread,
+#'   objective = "multi:softprob",
+#'   num_class = nclass,
+#'   verbose = 0
+#' )
+#' trees0 <- seq(from = 0, by = nclass, length.out = nrounds)
 #' col <- rgb(0, 0, 1, 0.5)
-#' xgb.plot.shap(x, model = mbst, trees = trees0, target_class = 0, top_n = 4,
-#'               n_col = 2, col = col, pch = 16, pch_NA = 17)
-#' xgb.plot.shap(x, model = mbst, trees = trees0 + 1, target_class = 1, top_n = 4,
-#'               n_col = 2, col = col, pch = 16, pch_NA = 17)
-#' xgb.plot.shap(x, model = mbst, trees = trees0 + 2, target_class = 2, top_n = 4,
-#'               n_col = 2, col = col, pch = 16, pch_NA = 17)
-#' xgb.ggplot.shap.summary(x, model = mbst, target_class = 0, top_n = 4)  # Summary plot
+#' xgb.plot.shap(
+#'   x,
+#'   model = mbst,
+#'   trees = trees0,
+#'   target_class = 0,
+#'   top_n = 4,
+#'   n_col = 2,
+#'   col = col,
+#'   pch = 16,
+#'   pch_NA = 17
+#' )
+#'
+#' xgb.plot.shap(
+#'   x,
+#'   model = mbst,
+#'   trees = trees0 + 1,
+#'   target_class = 1,
+#'   top_n = 4,
+#'   n_col = 2,
+#'   col = col,
+#'   pch = 16,
+#'   pch_NA = 17
+#' )
+#'
+#' xgb.plot.shap(
+#'   x,
+#'   model = mbst,
+#'   trees = trees0 + 2,
+#'   target_class = 2,
+#'   top_n = 4,
+#'   n_col = 2,
+#'   col = col,
+#'   pch = 16,
+#'   pch_NA = 17
+#' )
+#'
+#' # Summary plot
+#' xgb.ggplot.shap.summary(x, model = mbst, target_class = 0, top_n = 4)
 #'
 #' @rdname xgb.plot.shap
 #' @export
@@ -183,41 +242,48 @@ xgb.plot.shap <- function(data, shap_contrib = NULL, features = NULL, top_n = 1,
   invisible(list(data = data, shap_contrib = shap_contrib))
 }
 
-#' SHAP contribution dependency summary plot
+#' SHAP summary plot
 #'
-#' Compare SHAP contributions of different features.
+#' Visualizes SHAP contributions of different features.
 #'
-#' A point plot (each point representing one sample from \code{data}) is
+#' A point plot (each point representing one observation from `data`) is
 #' produced for each feature, with the points plotted on the SHAP value axis.
-#' Each point (observation) is coloured based on its feature value. The plot
-#' hence allows us to see which features have a negative / positive contribution
+#' Each point (observation) is coloured based on its feature value.
+#'
+#' The plot allows to see which features have a negative / positive contribution
 #' on the model prediction, and whether the contribution is different for larger
-#' or smaller values of the feature. We effectively try to replicate the
-#' \code{summary_plot} function from https://github.com/shap/shap.
+#' or smaller values of the feature. Inspired by the summary plot of
+#' <https://github.com/shap/shap>.
 #'
 #' @inheritParams xgb.plot.shap
 #'
-#' @return A \code{ggplot2} object.
+#' @return A `ggplot2` object.
 #' @export
 #'
-#' @examples # See \code{\link{xgb.plot.shap}}.
-#' @seealso \code{\link{xgb.plot.shap}}, \code{\link{xgb.ggplot.shap.summary}},
-#'   \url{https://github.com/shap/shap}
+#' @examples
+#' # See examples in xgb.plot.shap()
+#'
+#' @seealso [xgb.plot.shap()], [xgb.ggplot.shap.summary()],
+#'   and the Python library <https://github.com/shap/shap>.
 xgb.plot.shap.summary <- function(data, shap_contrib = NULL, features = NULL, top_n = 10, model = NULL,
                                   trees = NULL, target_class = NULL, approxcontrib = FALSE, subsample = NULL) {
   # Only ggplot implementation is available.
   xgb.ggplot.shap.summary(data, shap_contrib, features, top_n, model, trees, target_class, approxcontrib, subsample)
 }
 
-#' Prepare data for SHAP plots. To be used in xgb.plot.shap, xgb.plot.shap.summary, etc.
-#' Internal utility function.
+#' Prepare data for SHAP plots
+#'
+#' Internal function used in [xgb.plot.shap()], [xgb.plot.shap.summary()], etc.
 #'
 #' @inheritParams xgb.plot.shap
+#' @param max_observations Maximum number of observations to consider.
 #' @keywords internal
+#' @noRd
 #'
-#' @return A list containing: 'data', a matrix containing sample observations
-#'   and their feature values; 'shap_contrib', a matrix containing the SHAP contribution
-#'   values for these observations.
+#' @return
+#' A list containing:
+#' - `data`: The matrix of feature values.
+#' - `shap_contrib`: The matrix with corresponding SHAP values.
 xgb.shap.data <- function(data, shap_contrib = NULL, features = NULL, top_n = 1, model = NULL,
                           trees = NULL, target_class = NULL, approxcontrib = FALSE,
                           subsample = NULL, max_observations = 100000) {
@@ -237,7 +303,11 @@ xgb.shap.data <- function(data, shap_contrib = NULL, features = NULL, top_n = 1,
   if (is.character(features) && is.null(colnames(data)))
     stop("either provide `data` with column names or provide `features` as column indices")
 
-  if (is.null(model$feature_names) && model$nfeatures != ncol(data))
+  model_feature_names <- NULL
+  if (is.null(features) && !is.null(model)) {
+    model_feature_names <- xgb.feature_names(model)
+  }
+  if (is.null(model_feature_names) && xgb.num_feature(model) != ncol(data))
     stop("if model has no feature_names, columns in `data` must match features in model")
 
   if (!is.null(subsample)) {
@@ -266,7 +336,7 @@ xgb.shap.data <- function(data, shap_contrib = NULL, features = NULL, top_n = 1,
   }
 
   if (is.null(features)) {
-    if (!is.null(model$feature_names)) {
+    if (!is.null(model_feature_names)) {
       imp <- xgb.importance(model = model, trees = trees)
     } else {
       imp <- xgb.importance(model = model, trees = trees, feature_names = colnames(data))
