@@ -2,13 +2,14 @@
 Use scikit-learn regressor interface with CPU histogram tree method
 ===================================================================
 """
+
 from dask import array as da
 from dask.distributed import Client, LocalCluster
 
-import xgboost
+from xgboost import dask as dxgb
 
 
-def main(client):
+def main(client: Client) -> dxgb.Booster:
     # generate some random data for demonstration
     n = 100
     m = 10000
@@ -16,7 +17,7 @@ def main(client):
     X = da.random.random((m, n), partition_size)
     y = da.random.random(m, partition_size)
 
-    regressor = xgboost.dask.DaskXGBRegressor(verbosity=1, n_estimators=2)
+    regressor = dxgb.DaskXGBRegressor(verbosity=1, n_estimators=2)
     regressor.set_params(tree_method="hist")
     # assigning client here is optional
     regressor.client = client
