@@ -53,7 +53,7 @@ namespace parameter {  \
 template <>  \
 class FieldEntry<EnumClass> : public FieldEntry<int> {  \
  public:  \
-  FieldEntry<EnumClass>() {  \
+  FieldEntry() {  \
     static_assert(  \
       std::is_same<int, typename std::underlying_type<EnumClass>::type>::value,  \
       "enum class must be backed by int");  \
