@@ -25,7 +25,7 @@ private[spark] trait LearningTaskParams extends Params {
   /**
    * Specify the learning task and the corresponding learning objective.
    * options: reg:squarederror, reg:squaredlogerror, reg:logistic, binary:logistic, binary:logitraw,
-   * count:poisson, multi:softmax, multi:softprob, rank:pairwise, reg:gamma.
+   * count:poisson, multi:softmax, multi:softprob, rank:ndcg, reg:gamma.
    * default: reg:squarederror
    */
   final val objective = new Param[String](this, "objective",
@@ -68,11 +68,13 @@ private[spark] trait LearningTaskParams extends Params {
   /**
    * Fraction of training points to use for testing.
    */
+  @Deprecated
   final val trainTestRatio = new DoubleParam(this, "trainTestRatio",
     "fraction of training points to use for testing",
     ParamValidators.inRange(0, 1))
   setDefault(trainTestRatio, 1.0)
 
+  @Deprecated
   final def getTrainTestRatio: Double = $(trainTestRatio)
 
   /**
@@ -112,8 +114,4 @@ private[spark] object LearningTaskParams {
 
   val supportedObjectiveType = HashSet("regression", "classification")
 
-  val evalMetricsToMaximize = HashSet("auc", "aucpr", "ndcg", "map")
-
-  val evalMetricsToMinimize = HashSet("rmse", "rmsle", "mae", "mape", "logloss", "error", "merror",
-    "mlogloss", "gamma-deviance")
 }
