@@ -1,15 +1,16 @@
 """XGBoost collective communication related API."""
+
 import ctypes
-import json
 import logging
+import os
 import pickle
 from enum import IntEnum, unique
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Optional
 
 import numpy as np
 
 from ._typing import _T
-from .core import _LIB, _check_call, c_str, from_pystr_to_cstr, py_str
+from .core import _LIB, _check_call, build_info, c_str, make_jcargs, py_str
 
 LOGGER = logging.getLogger("[xgboost.collective]")
 
@@ -19,49 +20,35 @@ def init(**args: Any) -> None:
 
     Parameters
     ----------
-    args: Dict[str, Any]
+    args :
         Keyword arguments representing the parameters and their values.
 
         Accepted parameters:
-          - xgboost_communicator: The type of the communicator. Can be set as an environment
-            variable.
+          - dmlc_communicator: The type of the communicator.
             * rabit: Use Rabit. This is the default if the type is unspecified.
             * federated: Use the gRPC interface for Federated Learning.
-        Only applicable to the Rabit communicator (these are case sensitive):
-          -- rabit_tracker_uri: Hostname of the tracker.
-          -- rabit_tracker_port: Port number of the tracker.
-          -- rabit_task_id: ID of the current task, can be used to obtain deterministic rank
-             assignment.
-          -- rabit_world_size: Total number of workers.
-          -- rabit_hadoop_mode: Enable Hadoop support.
-          -- rabit_tree_reduce_minsize: Minimal size for tree reduce.
-          -- rabit_reduce_ring_mincount: Minimal count to perform ring reduce.
-          -- rabit_reduce_buffer: Size of the reduce buffer.
-          -- rabit_bootstrap_cache: Size of the bootstrap cache.
-          -- rabit_debug: Enable debugging.
-          -- rabit_timeout: Enable timeout.
-          -- rabit_timeout_sec: Timeout in seconds.
-          -- rabit_enable_tcp_no_delay: Enable TCP no delay on Unix platforms.
-        Only applicable to the Rabit communicator (these are case-sensitive, and can be set as
-        environment variables):
-          -- DMLC_TRACKER_URI: Hostname of the tracker.
-          -- DMLC_TRACKER_PORT: Port number of the tracker.
-          -- DMLC_TASK_ID: ID of the current task, can be used to obtain deterministic rank
-             assignment.
-          -- DMLC_ROLE: Role of the current task, "worker" or "server".
-          -- DMLC_NUM_ATTEMPT: Number of attempts after task failure.
-          -- DMLC_WORKER_CONNECT_RETRY: Number of retries to connect to the tracker.
-        Only applicable to the Federated communicator (use upper case for environment variables, use
-        lower case for runtime configuration):
-          -- federated_server_address: Address of the federated server.
-          -- federated_world_size: Number of federated workers.
-          -- federated_rank: Rank of the current worker.
-          -- federated_server_cert: Server certificate file path. Only needed for the SSL mode.
-          -- federated_client_key: Client key file path. Only needed for the SSL mode.
-          -- federated_client_cert: Client certificate file path. Only needed for the SSL mode.
+
+        Only applicable to the Rabit communicator:
+          - dmlc_tracker_uri: Hostname of the tracker.
+          - dmlc_tracker_port: Port number of the tracker.
+          - dmlc_task_id: ID of the current task, can be used to obtain deterministic
+          - dmlc_retry: The number of retry when handling network errors.
+          - dmlc_timeout: Timeout in seconds.
+          - dmlc_nccl_path: Path to load (dlopen) nccl for GPU-based communication.
+
+        Only applicable to the Federated communicator (use upper case for environment
+        variables, use lower case for runtime configuration):
+
+          - federated_server_address: Address of the federated server.
+          - federated_world_size: Number of federated workers.
+          - federated_rank: Rank of the current worker.
+          - federated_server_cert: Server certificate file path. Only needed for the SSL
+            mode.
+          - federated_client_key: Client key file path. Only needed for the SSL mode.
+          - federated_client_cert: Client certificate file path. Only needed for the SSL
+            mode.
     """
-    config = from_pystr_to_cstr(json.dumps(args))
-    _check_call(_LIB.XGCommunicatorInit(config))
+    _check_call(_LIB.XGCommunicatorInit(make_jcargs(**args)))
 
 
 def finalize() -> None:
@@ -155,7 +142,7 @@ def broadcast(data: _T, root: int) -> _T:
         assert data is not None, "need to pass in data when broadcasting"
         s = pickle.dumps(data, protocol=pickle.HIGHEST_PROTOCOL)
         length.value = len(s)
-    # run first broadcast
+    # Run first broadcast
     _check_call(
         _LIB.XGCommunicatorBroadcast(
             ctypes.byref(length), ctypes.sizeof(ctypes.c_ulong), root
@@ -182,16 +169,29 @@ def broadcast(data: _T, root: int) -> _T:
 
 
 # enumeration of dtypes
-DTYPE_ENUM__ = {
-    np.dtype("int8"): 0,
-    np.dtype("uint8"): 1,
-    np.dtype("int32"): 2,
-    np.dtype("uint32"): 3,
-    np.dtype("int64"): 4,
-    np.dtype("uint64"): 5,
-    np.dtype("float32"): 6,
-    np.dtype("float64"): 7,
-}
+def _map_dtype(dtype: np.dtype) -> int:
+    dtype_map = {
+        np.dtype("float16"): 0,
+        np.dtype("float32"): 1,
+        np.dtype("float64"): 2,
+        np.dtype("int8"): 4,
+        np.dtype("int16"): 5,
+        np.dtype("int32"): 6,
+        np.dtype("int64"): 7,
+        np.dtype("uint8"): 8,
+        np.dtype("uint16"): 9,
+        np.dtype("uint32"): 10,
+        np.dtype("uint64"): 11,
+    }
+    try:
+        dtype_map.update({np.dtype("float128"): 3})
+    except TypeError:  # float128 doesn't exist on the system
+        pass
+
+    if dtype not in dtype_map:
+        raise TypeError(f"data type {dtype} is not supported on the current platform.")
+
+    return dtype_map[dtype]
 
 
 @unique
@@ -227,29 +227,53 @@ def allreduce(data: np.ndarray, op: Op) -> np.ndarray:  # pylint:disable=invalid
     """
     if not isinstance(data, np.ndarray):
         raise TypeError("allreduce only takes in numpy.ndarray")
-    buf = data.ravel()
-    if buf.base is data.base:
-        buf = buf.copy()
-    if buf.dtype not in DTYPE_ENUM__:
-        raise TypeError(f"data type {buf.dtype} not supported")
+    buf = data.ravel().copy()
     _check_call(
         _LIB.XGCommunicatorAllreduce(
             buf.ctypes.data_as(ctypes.c_void_p),
             buf.size,
-            DTYPE_ENUM__[buf.dtype],
+            _map_dtype(buf.dtype),
             int(op),
-            None,
-            None,
         )
     )
     return buf
 
 
+def signal_error() -> None:
+    """Kill the process."""
+    _check_call(_LIB.XGCommunicatorSignalError())
+
+
 class CommunicatorContext:
     """A context controlling collective communicator initialization and finalization."""
 
     def __init__(self, **args: Any) -> None:
         self.args = args
+        key = "dmlc_nccl_path"
+        if args.get(key, None) is not None:
+            return
+
+        binfo = build_info()
+        if not binfo["USE_DLOPEN_NCCL"]:
+            return
+
+        try:
+            # PyPI package of NCCL.
+            from nvidia.nccl import lib
+
+            # There are two versions of nvidia-nccl, one is from PyPI, another one from
+            # nvidia-pyindex. We support only the first one as the second one is too old
+            # (2.9.8 as of writing).
+            if lib.__file__ is not None:
+                dirname: Optional[str] = os.path.dirname(lib.__file__)
+            else:
+                dirname = None
+
+            if dirname:
+                path = os.path.join(dirname, "libnccl.so.2")
+                self.args[key] = path
+        except ImportError:
+            pass
 
     def __enter__(self) -> Dict[str, Any]:
         init(**self.args)
