@@ -1,22 +1,21 @@
 /*!
- * Copyright 2018 XGBoost contributors
+ * Copyright 2018-2022 XGBoost contributors
  */
 #ifndef XGBOOST_COMMON_TRANSFORM_H_
 #define XGBOOST_COMMON_TRANSFORM_H_
 
-#include <dmlc/omp.h>
 #include <dmlc/common.h>
-
+#include <dmlc/omp.h>
 #include <xgboost/data.h>
+
+#include <type_traits>  // enable_if
 #include <utility>
 #include <vector>
-#include <type_traits>  // enable_if
-
-#include "xgboost/host_device_vector.h"
-#include "xgboost/span.h"
 
 #include "common.h"
 #include "threading_utils.h"
+#include "xgboost/host_device_vector.h"
+#include "xgboost/span.h"
 
 #if defined (__CUDACC__)
 #include "device_helpers.cuh"
@@ -61,10 +60,8 @@ class Transform {
   template <typename Functor>
   struct Evaluator {
    public:
-    Evaluator(Functor func, Range range, int device, bool shard) :
-        func_(func), range_{std::move(range)},
-        shard_{shard},
-        device_{device} {}
+    Evaluator(Functor func, Range range, int32_t n_threads, DeviceOrd device)
+        : func_(func), range_{std::move(range)}, n_threads_{n_threads}, device_{device} {}
 
     /*!
      * \brief Evaluate the functor with input pointers to HostDeviceVector.
@@ -74,7 +71,7 @@ class Transform {
      */
     template <typename... HDV>
     void Eval(HDV... vectors) const {
-      bool on_device = device_ >= 0;
+      bool on_device = device_.IsCUDA();
 
       if (on_device) {
         LaunchCUDA(func_, vectors...);
@@ -119,11 +116,11 @@ class Transform {
     }
     // Recursive unpack for Shard.
     template <typename T>
-    void UnpackShard(int device, const HostDeviceVector<T> *vector) const {
+    void UnpackShard(DeviceOrd device, const HostDeviceVector<T> *vector) const {
       vector->SetDevice(device);
     }
     template <typename Head, typename... Rest>
-    void UnpackShard(int device,
+    void UnpackShard(DeviceOrd device,
                      const HostDeviceVector<Head> *_vector,
                      const HostDeviceVector<Rest> *... _vectors) const {
       _vector->SetDevice(device);
@@ -134,9 +131,7 @@ class Transform {
     template <typename std::enable_if<CompiledWithCuda>::type* = nullptr,
               typename... HDV>
     void LaunchCUDA(Functor _func, HDV*... _vectors) const {
-      if (shard_) {
-        UnpackShard(device_, _vectors...);
-      }
+      UnpackShard(device_, _vectors...);
 
       size_t range_size = *range_.end() - *range_.begin();
 
@@ -145,7 +140,7 @@ class Transform {
       // granularity is used in data vector.
       size_t shard_size = range_size;
       Range shard_range {0, static_cast<Range::DifferenceType>(shard_size)};
-      dh::safe_cuda(cudaSetDevice(device_));
+      dh::safe_cuda(cudaSetDevice(device_.ordinal));
       const int kGrids =
           static_cast<int>(DivRoundUp(*(range_.end()), kBlockThreads));
       if (kGrids == 0) {
@@ -167,12 +162,10 @@ class Transform {
 #endif  // defined(__CUDACC__)
 
     template <typename... HDV>
-    void LaunchCPU(Functor func, HDV*... vectors) const {
+    void LaunchCPU(Functor func, HDV *...vectors) const {
       omp_ulong end = static_cast<omp_ulong>(*(range_.end()));
       SyncHost(vectors...);
-      ParallelFor(end, [&](omp_ulong idx) {
-        func(idx, UnpackHDV(vectors)...);
-      });
+      ParallelFor(end, n_threads_, [&](omp_ulong idx) { func(idx, UnpackHDV(vectors)...); });
     }
 
    private:
@@ -180,9 +173,8 @@ class Transform {
     Functor func_;
     /*! \brief Range object specifying parallel threads index range. */
     Range range_;
-    /*! \brief Whether sharding for vectors is required. */
-    bool shard_;
-    int device_;
+    int32_t n_threads_;
+    DeviceOrd device_;
   };
 
  public:
@@ -195,14 +187,13 @@ class Transform {
    * \param func    A callable object, accepting a size_t thread index,
    *                  followed by a set of Span classes.
    * \param range   Range object specifying parallel threads index range.
-   * \param device  Specify GPU to use.
-   * \param shard Whether Shard for HostDeviceVector is needed.
+   * \param n_threads  Number of CPU threads
+   * \param device_idx GPU device ordinal
    */
   template <typename Functor>
-  static Evaluator<Functor> Init(Functor func, Range const range,
-                                 int device,
-                                 bool const shard = true) {
-    return Evaluator<Functor> {func, std::move(range), device, shard};
+  static Evaluator<Functor> Init(Functor func, Range const range, int32_t n_threads,
+                                 DeviceOrd device) {
+    return Evaluator<Functor>{func, std::move(range), n_threads, device};
   }
 };
 
