@@ -1,5 +1,5 @@
 /**
- * Copyright 2021-2023, XGBoost contributors
+ * Copyright 2021-2024, XGBoost contributors
  */
 #ifndef XGBOOST_DATA_HISTOGRAM_CUT_FORMAT_H_
 #define XGBOOST_DATA_HISTOGRAM_CUT_FORMAT_H_
@@ -23,6 +23,15 @@ inline bool ReadHistogramCuts(common::HistogramCuts *cuts, common::AlignedResour
   if (!common::ReadVec(fi, &cuts->min_vals_.HostVector())) {
     return false;
   }
+  bool has_cat{false};
+  if (!fi->Read(&has_cat)) {
+    return false;
+  }
+  decltype(cuts->MaxCategory()) max_cat{0};
+  if (!fi->Read(&max_cat)) {
+    return false;
+  }
+  cuts->SetCategorical(has_cat, max_cat);
   return true;
 }
 
@@ -32,6 +41,8 @@ inline std::size_t WriteHistogramCuts(common::HistogramCuts const &cuts,
   bytes += common::WriteVec(fo, cuts.Values());
   bytes += common::WriteVec(fo, cuts.Ptrs());
   bytes += common::WriteVec(fo, cuts.MinValues());
+  bytes += fo->Write(cuts.HasCategorical());
+  bytes += fo->Write(cuts.MaxCategory());
   return bytes;
 }
 }  // namespace xgboost::data
