@@ -2,13 +2,13 @@
 
 set -euo pipefail
 
-WHEEL_TAG=manylinux2014_aarch64
+WHEEL_TAG=manylinux_2_28_aarch64
 
 echo "--- Build CPU code targeting ARM64"
 
 source tests/buildkite/conftest.sh
 
-command_wrapper="tests/ci_build/ci_build.sh aarch64 docker"
+command_wrapper="tests/ci_build/ci_build.sh aarch64"
 
 echo "--- Build libxgboost from the source"
 $command_wrapper tests/ci_build/build_via_cmake.sh --conda-env=aarch64_test \
@@ -19,13 +19,17 @@ $command_wrapper bash -c "cd build && ctest --extra-verbose"
 echo "--- Build binary wheel"
 $command_wrapper bash -c \
   "cd python-package && rm -rf dist/* && pip wheel --no-deps -v . --wheel-dir dist/"
-$command_wrapper python tests/ci_build/rename_whl.py python-package/dist/*.whl \
-  ${BUILDKITE_COMMIT} ${WHEEL_TAG}
+$command_wrapper python tests/ci_build/rename_whl.py  \
+  --wheel-path python-package/dist/*.whl  \
+  --commit-hash ${BUILDKITE_COMMIT}  \
+  --platform-tag ${WHEEL_TAG}
 
-echo "--- Audit binary wheel to ensure it's compliant with manylinux2014 standard"
+echo "--- Audit binary wheel to ensure it's compliant with ${WHEEL_TAG} standard"
 $command_wrapper auditwheel repair --plat ${WHEEL_TAG} python-package/dist/*.whl
-$command_wrapper python tests/ci_build/rename_whl.py wheelhouse/*.whl \
-  ${BUILDKITE_COMMIT} ${WHEEL_TAG}
+$command_wrapper python tests/ci_build/rename_whl.py  \
+  --wheel-path wheelhouse/*.whl  \
+  --commit-hash ${BUILDKITE_COMMIT}  \
+  --platform-tag ${WHEEL_TAG}
 mv -v wheelhouse/*.whl python-package/dist/
 # Make sure that libgomp.so is vendored in the wheel
 $command_wrapper bash -c \
