@@ -22,7 +22,7 @@ command_wrapper="tests/ci_build/ci_build.sh gpu_build_rockylinux8 --build-arg "`
 
 echo "--- Build libxgboost from the source"
 $command_wrapper tests/ci_build/build_via_cmake.sh \
-		 -DCMAKE_PREFIX_PATH="/opt/grpc;/opt/rmm" \
+		 -DCMAKE_PREFIX_PATH="/opt/grpc;/opt/rmm;/opt/rmm/lib64/rapids/cmake" \
 		 -DUSE_CUDA=ON \
 		 -DUSE_OPENMP=ON \
 		 -DHIDE_CXX_SYMBOLS=ON \
@@ -42,7 +42,7 @@ $command_wrapper python tests/ci_build/rename_whl.py  \
   --platform-tag ${WHEEL_TAG}
 
 echo "--- Audit binary wheel to ensure it's compliant with ${WHEEL_TAG} standard"
-tests/ci_build/ci_build.sh auditwheel_x86_64 auditwheel repair \
+tests/ci_build/ci_build.sh manylinux_2_28_x86_64 auditwheel repair \
   --plat ${WHEEL_TAG} python-package/dist/*.whl
 $command_wrapper python tests/ci_build/rename_whl.py  \
   --wheel-path wheelhouse/*.whl  \
@@ -50,7 +50,7 @@ $command_wrapper python tests/ci_build/rename_whl.py  \
   --platform-tag ${WHEEL_TAG}
 mv -v wheelhouse/*.whl python-package/dist/
 # Make sure that libgomp.so is vendored in the wheel
-tests/ci_build/ci_build.sh auditwheel_x86_64 bash -c \
+tests/ci_build/ci_build.sh manylinux_2_28_x86_64 bash -c \
   "unzip -l python-package/dist/*.whl | grep libgomp  || exit -1"
 
 echo "--- Upload Python wheel"
