@@ -4,8 +4,8 @@ set -euo pipefail
 
 source tests/buildkite/conftest.sh
 
-echo "--- Build XGBoost JVM packages scala 2.12"
-tests/ci_build/ci_build.sh jvm docker tests/ci_build/build_jvm_packages.sh \
+echo "--- Build and test XGBoost JVM packages with Scala 2.12"
+tests/ci_build/ci_build.sh jvm tests/ci_build/build_jvm_packages.sh \
   ${SPARK_VERSION}
 
 echo "--- Stash XGBoost4J JARs (Scala 2.12)"
@@ -14,9 +14,9 @@ buildkite-agent artifact upload "jvm-packages/xgboost4j-spark/target/*.jar"
 buildkite-agent artifact upload "jvm-packages/xgboost4j-flink/target/*.jar"
 buildkite-agent artifact upload "jvm-packages/xgboost4j-example/target/*.jar"
 
-echo "--- Build XGBoost JVM packages scala 2.13"
+echo "--- Build and test XGBoost JVM packages with Scala 2.13"
 
-tests/ci_build/ci_build.sh jvm docker tests/ci_build/build_jvm_packages.sh \
+tests/ci_build/ci_build.sh jvm tests/ci_build/build_jvm_packages.sh \
   ${SPARK_VERSION} "" "" "true"
 
 echo "--- Stash XGBoost4J JARs (Scala 2.13)"
