@@ -1,3 +1,5 @@
+import os
+import tempfile
 import weakref
 from typing import Any, Callable, Dict, List
 
@@ -50,6 +52,21 @@ def test_single_batch(tree_method: str = "approx") -> None:
     assert from_np.get_dump() == from_it.get_dump()
 
 
+def test_with_cat_single() -> None:
+    X, y = tm.make_categorical(
+        n_samples=128, n_features=3, n_categories=6, onehot=False
+    )
+    Xy = xgb.DMatrix(SingleBatch(data=X, label=y), enable_categorical=True)
+    from_it = xgb.train({}, Xy, num_boost_round=3)
+
+    Xy = xgb.DMatrix(X, y, enable_categorical=True)
+    from_Xy = xgb.train({}, Xy, num_boost_round=3)
+
+    jit = from_it.save_raw(raw_format="json")
+    jxy = from_Xy.save_raw(raw_format="json")
+    assert jit == jxy
+
+
 def run_data_iterator(
     n_samples_per_batch: int,
     n_features: int,
@@ -158,9 +175,11 @@ def test_data_iterator(
 
 
 class IterForCacheTest(xgb.DataIter):
-    def __init__(self, x: np.ndarray, y: np.ndarray, w: np.ndarray) -> None:
+    def __init__(
+        self, x: np.ndarray, y: np.ndarray, w: np.ndarray, release_data: bool
+    ) -> None:
         self.kwargs = {"data": x, "label": y, "weight": w}
-        super().__init__(release_data=False)
+        super().__init__(release_data=release_data)
 
     def next(self, input_data: Callable) -> int:
         if self.it == 1:
@@ -179,7 +198,9 @@ def test_data_cache() -> None:
     n_samples_per_batch = 16
     data = make_batches(n_samples_per_batch, n_features, n_batches, False)
     batches = [v[0] for v in data]
-    it = IterForCacheTest(*batches)
+
+    # Test with a cache.
+    it = IterForCacheTest(batches[0], batches[1], batches[2], release_data=False)
     transform = xgb.data._proxy_transform
 
     called = 0
@@ -194,4 +215,46 @@ def test_data_cache() -> None:
     assert it._data_ref is weakref.ref(batches[0])
     assert called == 1
 
+    # Test without a cache.
+    called = 0
+    it = IterForCacheTest(batches[0], batches[1], batches[2], release_data=True)
+    xgb.QuantileDMatrix(it)
+    assert called == 4
+
     xgb.data._proxy_transform = transform
+
+
+def test_cat_check() -> None:
+    n_batches = 3
+    n_features = 2
+    n_samples_per_batch = 16
+
+    batches = []
+
+    for i in range(n_batches):
+        X, y = tm.make_categorical(
+            n_samples=n_samples_per_batch,
+            n_features=n_features,
+            n_categories=3,
+            onehot=False,
+        )
+        batches.append((X, y))
+
+    X, y = list(zip(*batches))
+    it = tm.IteratorForTest(X, y, None, cache=None)
+    Xy: xgb.DMatrix = xgb.QuantileDMatrix(it, enable_categorical=True)
+
+    with pytest.raises(ValueError, match="categorical features"):
+        xgb.train({"tree_method": "exact"}, Xy)
+
+    Xy = xgb.DMatrix(X[0], y[0], enable_categorical=True)
+    with pytest.raises(ValueError, match="categorical features"):
+        xgb.train({"tree_method": "exact"}, Xy)
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        cache_path = os.path.join(tmpdir, "cache")
+
+        it = tm.IteratorForTest(X, y, None, cache=cache_path)
+        Xy = xgb.DMatrix(it, enable_categorical=True)
+        with pytest.raises(ValueError, match="categorical features"):
+            xgb.train({"booster": "gblinear"}, Xy)
