@@ -202,6 +202,12 @@ def test_unbiased() -> None:
     # less biased on low ranks.
     assert df["ti+"].iloc[-1] < df["ti+"].iloc[0]
 
+    # Training continuation
+    ltr.fit(x, c, qid=q, eval_set=[(x, c)], eval_qid=[q], xgb_model=ltr)
+    # normalized
+    np.testing.assert_allclose(df["ti+"].iloc[0], 1.0)
+    np.testing.assert_allclose(df["tj-"].iloc[0], 1.0)
+
 
 def test_normalization() -> None:
     run_normalization("cpu")
