@@ -5,9 +5,13 @@ import pytest
 
 from xgboost import testing as tm
 
+pytestmark = [
+    pytest.mark.skipif(**tm.no_dask()),
+    pytest.mark.skipif(**tm.no_dask_cuda()),
+    tm.timeout(60),
+]
+
 
-@pytest.mark.skipif(**tm.no_dask())
-@pytest.mark.skipif(**tm.no_dask_cuda())
 @pytest.mark.skipif(**tm.no_cupy())
 @pytest.mark.mgpu
 def test_dask_training():
@@ -16,8 +20,6 @@ def test_dask_training():
     subprocess.check_call(cmd)
 
 
-@pytest.mark.skipif(**tm.no_dask_cuda())
-@pytest.mark.skipif(**tm.no_dask())
 @pytest.mark.mgpu
 def test_dask_sklearn_demo():
     script = os.path.join(tm.demo_dir(__file__), "dask", "sklearn_gpu_training.py")
